import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
import pandas as pd
import time
import os
from scipy import optimize
from scipy.integrate import simpson
import math

try:
    import plate_bending_cpp
except:
    from KIB_LAP.Plattentragwerke import plate_bending_cpp


class PlateBendingKirchhoffClass:
    def __init__(
        self,
        E,
        t,
        a,
        b,
        p0,
        x0,
        u,
        y0,
        v,
        nu=0.0,
        kappa_s=5 / 6,
        K=0,
        n_inte=50,
        loading="Regular",
        support="hhhh",
        reihen=8,
    ):
        """
        Initialisierung der Klasse PlateBendingKirchhoff.

        Args:
            E (float): Elastizitätsmodul.
            t (float): Dicke der Platte.
            a (float): Länge der Platte in x-Richtung.
            b (float): Länge der Platte in y-Richtung.
            p0 (float): Maximale Last.
            x0 (float): Startpunkt der Last in x-Richtung.
            u (float): Breite der Last in x-Richtung.
            y0 (float): Startpunkt der Last in y-Richtung.
            v (float): Breite der Last in y-Richtung.
            nu (float, optional): Querdehnzahl. Standardmäßig 0.0.
            kappa_s (float, optional): Schubkorrekturfaktor. Standardmäßig 5/6.
            K (int, optional): Torsionssteifigkeit. Standardmäßig 0.
            n_inte (int, optional): Anzahl der Integrationspunkte. Standardmäßig 50.
            loading (str, optional): Art der Belastung. Standardmäßig "Regular". Andere Eingabewerte Liste
            support (str, optional): Lagerungsbedingungen. Standardmäßig "hhhh".
            reihen (int, optional): Anzahl der Reihen für die Reihenentwicklung. Standardmäßig 8.
        """
        # Materialien
        self.E = E
        self.nu = nu
        self.kappa_s = kappa_s
        self.K = K
        # Plattenabmessungen in [m]
        self.t = t
        self.a = a
        self.b = b

        self.n_inte = n_inte

        self.list_a = np.linspace(0, self.a, self.n_inte)
        self.list_b = np.linspace(0, self.b, self.n_inte)

        # Belastung
        self.p0 = p0
        self.x0 = x0
        self.u = u
        self.y0 = y0
        self.v = v

        self.loading = loading

        # Steifigkeitsmatrix-Komponenten
        self.D_11 = E * t**3 / (12 * (1 - nu**2))
        self.D_22 = self.D_11
        self.D_12 = self.D_11 * nu
        self.D_66 = (1 - nu) / 2 * self.D_11

        self.support = support

        self.reihen = reihen

        self.mat = np.zeros((self.reihen**2, self.reihen**2))

        self.load = np.zeros((self.reihen**2))

    def CalculateAll(self):
        """
        Führt alle Berechnungsschritte aus.
        """
        self.AssembleStiffnessMatrix()
        self.Construct_Loadvector()
        self.ReduceMatrix()
        self.SolveSystem()
        self.TransformSolutionMatrix()

    def AssembleStiffnessMatrix(self):
        """
        Berechnung der Steifigkeitsmatrix für die Plattenstruktur mit dem C++-Modul via pybind11.
        """
        # Sicherstellen, dass das C++-Modul importiert werden kann
        try:
            pass
        except ImportError as e:
            print("Fehler beim Import des C++-Moduls:", e)
            raise
        print(self.support)
        # Aufruf der assemble_stiffness_matrix-Funktion aus dem C++-Modul
        self.mat = plate_bending_cpp.assemble_stiffness_matrix(
            self.D_11,
            self.D_22,
            self.D_12,
            self.D_66,
            self.reihen,
            self.n_inte,
            self.a,
            self.b,
            self.support,
        )

        self.mat = np.array(self.mat)  # Konvertierung in ein NumPy-Array

    def Construct_Loadvector(self):
        """
        Konstruktion des Lastvektors basierend auf der aufgebrachten Belastung.
        """
        if self.loading == "Regular":
            rectangular_loads = [
                # [x0, x1, y0, y1, p0]
                [self.x0, self.x0 + self.u, self.y0, self.y0 + self.v, self.p0]
            ]

            for load in rectangular_loads:
                x0, x1, y0, y1, p0 = load

                # Sicherstellen, dass die Last innerhalb der Plattenabmessungen liegt
                x0 = max(0, min(self.a, x0))
                x1 = max(0, min(self.a, x1))
                y0 = max(0, min(self.b, y0))
                y1 = max(0, min(self.b, y1))

                list_load_inte_x = np.linspace(x0, x1, 100)
                list_load_inte_y = np.linspace(y0, y1, 100)
                for m in range(1, self.reihen + 1):
                    for n in range(1, self.reihen + 1):
                        # Integration über die Lastfläche

                        y_values_1 = self.function_1(list_load_inte_x, m)
                        omega_1m = simpson(y_values_1, x=list_load_inte_x)

                        y_values_2 = self.function_2(list_load_inte_y, n)
                        omega_1n = simpson(y_values_2, x=list_load_inte_y)

                        self.load[n - 1 + self.reihen * (m - 1)] += (
                            p0 * omega_1m * omega_1n
                        )
        else:
            self.RectangularLoad = pd.DataFrame(
                pd.read_csv("Loading/Constant_Loading.csv")
            )
            for i in range(0, len(self.RectangularLoad["x0[m]"]), 1):
                x0 = float(self.RectangularLoad["x0[m]"][i])
                if (x0 < 0) or (x0 > self.a):
                    x0 = 0
                x1 = float(self.RectangularLoad["x1[m]"][i])
                if x1 > self.a or x1 < 0:
                    x1 = self.a
                y0 = float(self.RectangularLoad["y0[m]"][i])
                if y0 < 0 or y0 > self.b:
                    y0 = 0
                y1 = float(self.RectangularLoad["y1[m]"][i])
                if y1 < 0 or y1 > self.b:
                    y1 = self.b

                p0 = float(self.RectangularLoad["p0[MN/m**2]"][i])

                list_load_inte_x = np.linspace(x0, x1, 100)
                list_load_inte_y = np.linspace(y0, y1, 100)
                for m in range(1, self.reihen + 1, 1):
                    for n in range(1, self.reihen + 1, 1):

                        y_values_1 = self.function_1(list_load_inte_x, m)
                        omega_1m = simpson(y_values_1, x=list_load_inte_x)

                        y_values_2 = self.function_2(list_load_inte_y, n)
                        omega_1n = simpson(y_values_2, x=list_load_inte_y)

                        if m == 1:
                            print("Numeric integration")
                            print(omega_1m)
                            print(omega_1n)

                        self.load[n - 1 + self.reihen * (m - 1)] += (
                            p0 * omega_1m * omega_1n
                        )

    def ReduceMatrix(self):
        """
        Reduziert die Matrix und den Lastvektor, um Nullzeilen und -spalten zu entfernen.
        """
        # Prüfen, welche Zeilen und Spalten nur Nullen enthalten
        non_zero_rows = ~np.all(self.mat == 0, axis=1)
        non_zero_cols = ~np.all(self.mat == 0, axis=0)

        # Reduziere die Matrix und den Vektor, um nur Nicht-Null Zeilen und Spalten zu behalten
        self.reduced_mat = self.mat[non_zero_rows, :][:, non_zero_cols]
        self.reduced_load = self.load[non_zero_rows]

    def SolveSystem(self):
        """
        Löst das reduzierte Gleichungssystem.
        """
        if self.reduced_mat.size > 0:
            self.x_reduced = np.linalg.solve(self.reduced_mat, self.reduced_load)
        else:
            print("Keine Lösung möglich, da das Gleichungssystem nur Nullen enthält.")

    def TransformSolutionMatrix(self):
        """
        Transformiert die Lösungsvektoren in eine Matrixform.
        """
        # Berechnen der Dimension der quadratischen Matrix
        n = int(np.sqrt(len(self.x_reduced)))

        if n**2 != len(self.x_reduced):
            raise ValueError(
                "Die Länge von x_reduced ist nicht das Quadrat einer ganzen Zahl"
            )

        # Umstrukturieren des Vektors x_reduced in eine quadratische n x n Matrix
        self.matrix = self.x_reduced.reshape(n, n)

    def SolutionPointDisp(self, a_sol, b_sol):
        """
        Berechnet die Durchbiegung an einem bestimmten Punkt (a_sol, b_sol).
        """
        x_disp = 0.0

        for m in range(1, self.reihen + 1):
            for n in range(1, self.reihen + 1):
                x_disp += (
                    self.matrix[m - 1][n - 1]
                    * self.function_1(a_sol, m)
                    * self.function_2(b_sol, n)
                )

        return x_disp

    def SolutionPointMomentx(self, a_sol, b_sol):
        """
        Berechnet den Biegemoment mxx an einem bestimmten Punkt (a_sol, b_sol).
        """
        mxx = 0.0

        for m in range(1, self.reihen + 1):
            for n in range(1, self.reihen + 1):
                mxx += self.matrix[m - 1][n - 1] * (
                    -self.D_11 * self.function_1xx(a_sol, m) * self.function_2(b_sol, n)
                    - self.D_12
                    * self.function_2yy(b_sol, n)
                    * self.function_1(a_sol, m)
                )

        return mxx

    def SolutionPointMomenty(self, a_sol, b_sol):
        """
        Berechnet den Biegemoment myy an einem bestimmten Punkt (a_sol, b_sol).
        """
        myy = 0.0

        for m in range(1, self.reihen + 1):
            for n in range(1, self.reihen + 1):
                myy += self.matrix[m - 1][n - 1] * (
                    -self.D_12 * self.function_1xx(a_sol, m) * self.function_2(b_sol, n)
                    - self.D_22
                    * self.function_2yy(b_sol, n)
                    * self.function_1(a_sol, m)
                )

        return myy

    def SolutionPointMomentxy(self, a_sol, b_sol):
        """
        Berechnet das Drillmoment Mxy an einem bestimmten Punkt (a_sol, b_sol).
        """
        mxy = 0.0

        for m in range(1, self.reihen + 1):
            for n in range(1, self.reihen + 1):
                mxy += (
                    -self.D_66
                    * self.matrix[m - 1][n - 1]
                    * (self.function_1x(a_sol, m) * self.function_2y(b_sol, n))
                )

        return mxy

    def SolutionPointShearForceX(self, a_sol, b_sol):
        """
        Berechnet die Querkraft Qx an einem bestimmten Punkt (a_sol, b_sol).
        """
        Qx = 0.0

        for m in range(1, self.reihen + 1):
            for n in range(1, self.reihen + 1):
                term1 = (
                    self.D_11 * self.function_1xxx(a_sol, m) * self.function_2(b_sol, n)
                )
                term2 = (
                    (self.D_12 + self.D_66)
                    * self.function_1x(a_sol, m)
                    * self.function_2yy(b_sol, n)
                )
                Qx += self.matrix[m - 1][n - 1] * (term1 + term2)

        return Qx

    def SolutionPointShearForceY(self, a_sol, b_sol):
        """
        Berechnet die Querkraft Qy an einem bestimmten Punkt (a_sol, b_sol).
        """
        Qy = 0.0

        for m in range(1, self.reihen + 1):
            for n in range(1, self.reihen + 1):
                term1 = (
                    self.D_22 * self.function_1(a_sol, m) * self.function_2yyy(b_sol, n)
                )
                term2 = (
                    (self.D_12 + self.D_66)
                    * self.function_1xx(a_sol, m)
                    * self.function_2y(b_sol, n)
                )
                Qy += self.matrix[m - 1][n - 1] * (term1 + term2)

        return Qy

    def function_1(self, x, m):
        if self.support == "hhhh":
            return np.sin(x * np.pi / self.a * m)
        elif self.support == "cccc":
            return 1 - np.cos(2 * m * np.pi * x / self.a)
        elif self.support == "hhff":

            lambda_m = (0.50 + m - 1) * np.pi
            if m == 2:
                lambda_m = 4.730041
            elif m == 3:
                lambda_m = 7.853205
            elif m == 4:
                lambda_m = 10.99561

            alpha = lambda_m / self.a

            a_j = (np.sinh(lambda_m) - np.sin(lambda_m)) / (
                np.cosh(lambda_m) - np.cos(lambda_m)
            )
            if m > 1:
                return (np.sin(alpha * x) + np.sinh(alpha * x)) / (
                    np.sin(lambda_m) - np.sinh(lambda_m)
                ) - a_j * (np.cosh(alpha * x) + np.cos(alpha * x)) / (
                    np.cos(lambda_m) - np.cosh(lambda_m)
                )
            else:
                return np.ones_like(x)
        elif self.support == "hhhf":
            lambda_m = (0.25 + m - 1) * np.pi
            if m == 2:
                lambda_m = 3.926602
            elif m == 3:
                lambda_m = 7.068582
            elif m == 4:
                lambda_m = 10.21018

            alpha = lambda_m / self.a
            if m > 1:
                return np.sin(alpha * x) + np.sinh(alpha * x) * np.sin(
                    lambda_m
                ) / np.sinh(lambda_m)
            else:
                return x / self.a
        else:
            return np.zeros_like(x)

    def function_2(self, y, n):
        if self.support == "hhhh":
            return np.sin(y * np.pi / self.b * n)
        elif self.support == "cccc":
            return 1 - np.cos(2 * n * np.pi * y / self.b)
        elif self.support == "hhff":
            return np.sin(y * np.pi / self.b * n)
        elif self.support == "hhhf":
            return np.sin(y * np.pi / self.b * n)
        else:
            return np.zeros_like(y)

    def function_1x(self, x, m):
        if self.support == "hhhh":
            return np.cos(x * np.pi / self.a * m) * np.pi / self.a * m
        elif self.support == "cccc":
            return 2 * np.pi * m * np.sin(2 * np.pi * m * x / self.a) / self.a
        elif self.support == "hhff":
            lambda_m = (0.50 + m) * np.pi
            if m == 2:
                lambda_m = 4.730041
            elif m == 3:
                lambda_m = 7.853205
            elif m == 4:
                lambda_m = 10.99561

            alpha = lambda_m / self.a
            a_j = (np.sinh(lambda_m) - np.sin(lambda_m)) / (
                np.cosh(lambda_m) - np.cos(lambda_m)
            )
            if m > 1:
                return alpha * (
                    (np.cos(alpha * x) + np.cosh(alpha * x))
                    / (np.sin(lambda_m) - np.sinh(lambda_m))
                    - a_j
                    * (np.sinh(alpha * x) - np.sin(alpha * x))
                    / (np.cos(lambda_m) - np.cosh(lambda_m))
                )
            else:
                return np.zeros_like(x)

        elif self.support == "hhhf":
            lambda_m = (0.25 + m - 1) * np.pi
            if m == 2:
                lambda_m = 3.926602
            elif m == 3:
                lambda_m = 7.068582
            elif m == 4:
                lambda_m = 10.21018

            alpha = lambda_m / self.a
            if m > 1:
                return alpha * (
                    np.cos(alpha * x)
                    + np.cosh(alpha * x) * np.sin(lambda_m) / np.sinh(lambda_m)
                )
            else:
                return 1 / self.a
        else:
            return np.zeros_like(x)

    def function_1xx(self, x, m):
        if self.support == "hhhh":
            return -np.sin(x * np.pi / self.a * m) * (np.pi / self.a * m) ** 2
        elif self.support == "cccc":
            return 4 * np.pi**2 * m**2 * np.cos(2 * np.pi * m * x / self.a) / self.a**2
        elif self.support == "hhff":
            lambda_m = (0.50 + m - 1) * np.pi
            if m == 2:
                lambda_m = 4.730041
            elif m == 3:
                lambda_m = 7.853205
            elif m == 4:
                lambda_m = 10.99561

            alpha = lambda_m / self.a
            a_j = (np.sinh(lambda_m) - np.sin(lambda_m)) / (
                np.cosh(lambda_m) - np.cos(lambda_m)
            )
            if m > 1:
                return (alpha**2) * (
                    (-np.sin(alpha * x) + np.sinh(alpha * x))
                    / (np.sin(lambda_m) - np.sinh(lambda_m))
                    - a_j
                    * (np.cosh(alpha * x) - np.cos(alpha * x))
                    / ((np.cos(lambda_m) - np.cosh(lambda_m)))
                )
            else:
                return np.zeros_like(x)

        elif self.support == "hhhf":
            lambda_m = (0.25 + m - 1) * np.pi
            if m == 2:
                lambda_m = 3.926602
            elif m == 3:
                lambda_m = 7.068582
            elif m == 4:
                lambda_m = 10.21018

            alpha = lambda_m / self.a
            if m > 1:
                return alpha**2 * (
                    -np.sin(alpha * x)
                    + np.sinh(alpha * x) * np.sin(lambda_m) / np.sinh(lambda_m)
                )
            else:
                return 0

        else:
            return np.zeros_like(x)

    def function_1xxx(self, x, m):
        if self.support == "hhhh":
            return np.cos(x * np.pi / self.a * m) * (np.pi / self.a * m) ** 3 * (-1)

        elif self.support == "cccc":
            return -8 * np.pi**3 * m**3 * np.sin(2 * np.pi * m * x / self.a) / self.a**3

        elif self.support == "hhff":
            lambda_m = (0.50 + m - 1) * np.pi
            if m == 2:
                lambda_m = 4.730041
            elif m == 3:
                lambda_m = 7.853205
            elif m == 4:
                lambda_m = 10.99561

            alpha = lambda_m / self.a
            a_j = (np.sinh(lambda_m) - np.sin(lambda_m)) / (
                np.cosh(lambda_m) - np.cos(lambda_m)
            )
            if m > 1:
                return (alpha**3) * (
                    (-np.cos(alpha * x) + np.cosh(alpha * x))
                    / (np.sin(lambda_m) - np.sinh(lambda_m))
                    - a_j
                    * (np.sinh(alpha * x) + np.sin(alpha * x))
                    / (np.cos(lambda_m) - np.cosh(lambda_m))
                )
            else:
                return np.zeros_like(x)  # Starrkörperverschiebung

        elif self.support == "hhhf":
            lambda_m = (0.25 + m - 1) * np.pi
            if m == 2:
                lambda_m = 3.926602
            elif m == 3:
                lambda_m = 7.068582
            elif m == 4:
                lambda_m = 10.21018

            alpha = lambda_m / self.a
            if m > 1:
                return alpha**3 * (
                    -np.cos(alpha * x)
                    + np.cosh(alpha * x) * np.sin(lambda_m) / np.sinh(lambda_m)
                )
            else:
                return 0
        else:
            return np.zeros_like(x)

    def function_2y(self, y, n):
        if self.support == "hhhh":
            return np.cos(y * np.pi / self.b * n) * np.pi / self.b * n
        elif self.support == "cccc":
            return 2 * np.pi * n * np.sin(2 * np.pi * n * y / self.b) / self.b
        elif self.support == "hhff":
            return 2 * np.pi * n * np.sin(2 * np.pi * n * y / self.b) / self.b
        elif self.support == "hhhf":
            return 2 * np.pi * n * np.sin(2 * np.pi * n * y / self.b) / self.b
        else:
            return np.zeros_like(x)

    def function_2yy(self, y, n):
        if self.support == "hhhh":
            return -np.sin(y * np.pi / self.b * n) * (np.pi / self.b * n) ** 2
        elif self.support == "cccc":
            return 4 * np.pi**2 * n**2 * np.cos(2 * np.pi * n * y / self.b) / self.b**2
        elif self.support == "hhff":
            return -np.sin(y * np.pi / self.b * n) * (np.pi / self.b * n) ** 2
        elif self.support == "hhhf":
            return -np.sin(y * np.pi / self.b * n) * (np.pi / self.b * n) ** 2
        else:
            return np.zeros_like(y)

    def function_2yyy(self, y, n):
        if self.support == "hhhh":
            return np.cos(y * np.pi / self.b * n) * (np.pi / self.b * n) ** 3 * (-1)
        elif self.support == "cccc":
            return -8 * np.pi**3 * n**3 * np.sin(2 * np.pi * n * y / self.b) / self.b**3
        elif self.support == "hhff":
            return np.cos(y * np.pi / self.b * n) * (np.pi / self.b * n) ** 3 * (-1)
        elif self.support == "hhhf":
            return np.cos(y * np.pi / self.b * n) * (np.pi / self.b * n) ** 3 * (-1)
        else:
            return np.zeros_like(y)

    def PlotLoad(self):
        """
        Plottet die aufgebrachte Belastung auf der Platte.
        """
        x_values = [0, self.a, self.a, 0, 0]
        y_values = [0, 0, self.b, self.b, 0]

        if self.loading == "Regular":
            rectangular_loads = [
                # [x0, x1, y0, y1]
                [self.x0, self.x0 + self.u, self.y0, self.y0 + self.v]
            ]

            for load in rectangular_loads:
                x0, x1, y0, y1 = load
                x = [x0, x1, x1, x0, x0]
                y = [y0, y0, y1, y1, y0]
                plt.plot(x, y)
        else:
            print("Else")
            for i in range(0, len(self.RectangularLoad["x0[m]"]), 1):
                x = [
                    self.RectangularLoad["x0[m]"][i],
                    self.RectangularLoad["x1[m]"][i],
                    self.RectangularLoad["x1[m]"][i],
                    self.RectangularLoad["x0[m]"][i],
                    self.RectangularLoad["x0[m]"][i],
                ]
                y = [
                    self.RectangularLoad["y0[m]"][i],
                    self.RectangularLoad["y0[m]"][i],
                    self.RectangularLoad["y1[m]"][i],
                    self.RectangularLoad["y1[m]"][i],
                    self.RectangularLoad["y0[m]"][i],
                ]

                plt.plot(x, y)

        plt.plot(x_values, y_values)
        plt.gca().set_aspect("equal", adjustable="box")
        plt.show(block=False)
        plt.pause(2)
        plt.close()

    def PlotDeflectionGrid(self, grid_size=20):
        """
        Berechnet und plottet ein Raster der Momentenverläufe.

        Args:
            grid_size (int): Anzahl der Unterteilungen in x- und y-Richtung. Standardmäßig 20.
        """
        x_values = np.linspace(0, self.a, grid_size)
        y_values = np.linspace(0, self.b, grid_size)
        self.w_values = np.zeros((grid_size, grid_size))

        for i, x in enumerate(x_values):
            for j, y in enumerate(y_values):
                self.w_values[i][j] = self.SolutionPointDisp(x, y)

        print("Max. Deflection", self.w_values.max())

        X, Y = np.meshgrid(x_values, y_values)
        fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
        surf = ax.plot_surface(
            X, Y, self.w_values, cmap=cm.coolwarm, linewidth=0, antialiased=False
        )
        ax.set_title("Deflections w in [mm]")
        plt.show(block=False)
        plt.pause(6)
        plt.close()

    def PlotMomentGrid(self, grid_size=20):
        """
        Berechnet und plottet ein Raster der Momentenverläufe.

        Args:
            grid_size (int): Anzahl der Unterteilungen in x- und y-Richtung. Standardmäßig 20.
        """
        x_values = np.linspace(0, self.a, grid_size)
        y_values = np.linspace(0, self.b, grid_size)
        self.mx_values = np.zeros((grid_size, grid_size))
        self.my_values = np.zeros((grid_size, grid_size))

        for i, x in enumerate(x_values):
            for j, y in enumerate(y_values):
                self.mx_values[i][j] = self.SolutionPointMomentx(x, y)
                self.my_values[i][j] = self.SolutionPointMomenty(x, y)

        print("Max-Moment x in [kNm]", self.mx_values.max() * 1000)
        print("Min-Moment x in [kNm]", self.mx_values.min() * 1000)
        print("Max-Moment y in [kNm]", self.my_values.max() * 1000)
        print("Min-Moment y in [kNm]", self.my_values.min() * 1000)

        X, Y = np.meshgrid(x_values, y_values)
        fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
        surf = ax.plot_surface(
            X, Y, self.mx_values, cmap=cm.coolwarm, linewidth=0, antialiased=False
        )
        ax.set_title("Moment Mx")
        plt.show(block=False)
        plt.pause(6)
        plt.close()

        fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
        surf = ax.plot_surface(
            X, Y, self.my_values, cmap=cm.coolwarm, linewidth=0, antialiased=False
        )
        ax.set_title("Moment My")
        plt.show(block=False)
        plt.pause(6)
        plt.close()

    def PlotShearForceGrid(self, grid_size=20):
        """
        Berechnet und plottet ein Raster der Querkräfte.
        """
        x_values = np.linspace(0, self.a, grid_size)
        y_values = np.linspace(0, self.b, grid_size)
        self.qx_values = np.zeros((grid_size, grid_size))
        self.qy_values = np.zeros((grid_size, grid_size))

        for i, x in enumerate(x_values):
            for j, y in enumerate(y_values):
                self.qx_values[i][j] = self.SolutionPointShearForceX(x, y)
                self.qy_values[i][j] = self.SolutionPointShearForceY(x, y)

        print("Max Qx in [kN/m]", self.qx_values.max() * 1000)
        print("Min Qx in [kN/m]", self.qx_values.min() * 1000)
        print("Max Qy in [kN/m]", self.qy_values.max() * 1000)
        print("Min Qy in [kN/m]", self.qy_values.min() * 1000)

        X, Y = np.meshgrid(x_values, y_values)
        fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
        surf = ax.plot_surface(
            X, Y, self.qx_values, cmap=cm.coolwarm, linewidth=0, antialiased=False
        )
        ax.set_title("Querkraft Qx")
        plt.show(block=False)
        plt.pause(6)
        plt.close()

        fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
        surf = ax.plot_surface(
            X, Y, self.qy_values, cmap=cm.coolwarm, linewidth=0, antialiased=False
        )
        ax.set_title("Querkraft Qy")
        plt.show(block=False)
        plt.pause(6)
        plt.close()

    def PlotTorsionalMomentGrid(self, grid_size=20):
        """
        Berechnet und plottet ein Raster der Drillmomente.
        """
        x_values = np.linspace(0, self.a, grid_size)
        y_values = np.linspace(0, self.b, grid_size)
        self.mxy_values = np.zeros((grid_size, grid_size))

        for i, x in enumerate(x_values):
            for j, y in enumerate(y_values):
                self.mxy_values[i][j] = self.SolutionPointMomentxy(x, y)

        print("Max Mxy in [kNm/m]", self.mxy_values.max() * 1000)
        print("Min Mxy in [kNm/m]", self.mxy_values.min() * 1000)

        X, Y = np.meshgrid(x_values, y_values)
        fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
        surf = ax.plot_surface(
            X, Y, self.mxy_values, cmap=cm.coolwarm, linewidth=0, antialiased=False
        )
        ax.set_title("Drillmoment Mxy")
        plt.show(block=False)
        plt.pause(6)
        plt.close()







import pandas as pd
import os
import sys
import numpy as np

# Absoluten Pfad des Projekts und des KIB-Ordners berechnen
project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
kib_directory = os.path.join(project_root, "KIB_LAP")

# Debug-Ausgabe, um die Pfade zu überprüfen
print("Project Root:", project_root)
print("KIB Directory:", kib_directory)

# Füge das KIB-Verzeichnis zum sys.path hinzu, falls es nicht vorhanden ist
if kib_directory not in sys.path:
    sys.path.insert(0, kib_directory)

# Importieren des Moduls
try:
    from KIB_LAP.Querschnittswerte import CrossSectionThin 
    print("Import erfolgreich!")
except ModuleNotFoundError as e:
    print("Fehler beim Import:", e)


Node_Cords = pd.DataFrame({
    "Nr.": [1, 2, 3, 4, 5, 6],
    "y": [-0.35, 0, 0.35, 0, -0.30, 0.30],
    "z": [0, 0, 0, 0.400, 0.400, 0.400]
})


CrossSectionElements = pd.DataFrame({
    "nr": [1, 2, 3, 4, 5],
    "npa": [1, 2, 2, 4, 4],
    "npe": [2, 3, 4, 5, 6],
    "t [m]": [0.02, 0.02, 0.01, 0.02, 0.02]
})

Class = CrossSectionThin(
    2.1e5,
    0.3,
    Node_Cords,
    CrossSectionElements,
    "Calculation"
)


Class.read_node_input()
Class.CalculateElementStiffness()
Class.Calculate_GesMat()
Class.SolverTorsion()
Class.CalculateAyzw()
Class.Update_SMP()
Class.Calculate_IwIt()
Class.Calculate_WoWu()
Class.Calculate_ShearStress_Vz()
Class.Calculate_imryrzrw()
Class.Export_Controll_Data()
Class.Export_Cross_Section_Data()


import numpy as np
from scipy.integrate import simpson as simps
import math


class BeamRectangular:
    def __init__(self, index, width, height):
        self.index = index
        self.width = width
        self.height = height
        self.cross_section_area = width * height
        self.center_of_gravity = height / 2
        self.cross_section_inertia = (width * height**3) / 12

    # Check if the beam is suitable for the bending calculation DIN EN 1992-1-1
    def check_if_beam_is_suitable_for_bending_calc(self, ed_i):
        check = ed_i / self.height
        return check

    # Bending Calculation after DIN EN 1992-1-1
    def calculate_beam_section(
        self, beam_sub_section, strength_of_concrete, tensile_yield_strength, nu_limit
    ):
        M = beam_sub_section.moment_ed / 1000000  # Mnm
        N_f = beam_sub_section.normal_force_ed / 1000000  # Mn

        Mrds = 0
        Meds = 0
        Meds_lim = 0

        nu_eds = 0
        nu_eds_grenz = nu_limit  # 0.371 # 0.296

        fcd = strength_of_concrete  # N/mm²
        fyd = tensile_yield_strength  # MN/m²
        E = beam_sub_section.elasticity_modulus_steel  # MN/m²
        fck = (fcd * 1.5) / 0.85
        fctm = 0.3 * fck ** (2 / 3)
        fyk = fyd * 1.15

        epssyd = (fyd / E) * 1000

        As1erf = 0  # cm²
        As2erf = 0  # cm² Druckbewehrung
        # Geometrie
        h = self.height  # gesamt Höhe
        b_w = self.width  # Balkenbreite

        zs1 = self.center_of_gravity  # Schwerpunkt

        d = beam_sub_section.effective_height
        d2 = beam_sub_section.effective_height_pressure

        # Startwerte der Dehnungen und Stauchungen
        epss = 25
        steal_strain_1 = 0  # Falls Druckbewehrung, hier die neue Stahldehnung
        epss2 = 0
        epsc = 0.0001  # Darf nicht 0 sein, da sonst Teilung durch 0 in der Berechnung der Schwerpunkte -> Start mit 0,001

        epss_new = 0
        epss_2_new = 0

        num_iter = 0
        num_iter_grenz = 30000

        N = 50  # Numerische Integration der Spannungen ueber die Hoehe

        xi = 0
        x = 0
        z = 0

        # KENNWERTE
        # Querschnittslängen
        interval_start_1 = 0
        interval_end_1 = 0
        interval_start_2 = 0
        interval_end_2 = 0
        interval_start_3 = 0
        interval_end_3 = 0
        # Querschnittsbreiten
        concrete_width_1 = 0
        concrete_width_2 = 0
        concrete_width_3 = 0
        # Druckkräfte und Schwerpunkte
        A_1 = 0
        S_1 = 0
        A_2 = 0
        S_2 = 0
        A_3 = 0
        S_3 = 0
        while Mrds <= abs(Meds) and num_iter < num_iter_grenz:
            xi = epsc / (epsc + epss)
            x = xi * d

            # Stahl fließt nicht
            if epss < epssyd:
                sigma_s = E * epss * 0.001
            # Stahl fließt
            else:
                sigma_s = fyd

            # Feldmoment
            if M >= 0:

                # Grenzdehnung von 2 wird nicht erreicht
                if 0 <= epsc < 2.0:
                    # Querschnittslängen Vergabe
                    interval_start_1 = 0
                    interval_end_1 = x
                    interval_start_2 = 0
                    interval_end_2 = 0
                    interval_start_3 = 0
                    interval_end_3 = 0
                    # Querschnittsbreiten Vergabe
                    concrete_width_1 = b_w
                    concrete_width_2 = 0
                    concrete_width_3 = 0
                # Grenzdehnung von 2 wird erreicht
                elif epsc >= 2.0:
                    # Querschnittslängen Vergabe
                    interval_start_1 = 0
                    interval_end_1 = (
                        2 / epsc * x
                    )  # 2 = Dehnung beim Erreichen der Druckfestigkeit
                    interval_start_2 = 2 / epsc * x
                    interval_end_2 = x
                    interval_start_3 = 0
                    interval_end_3 = 0
                    # Querschnittsbreiten Vergabe
                    concrete_width_1 = b_w
                    concrete_width_2 = b_w
                    concrete_width_3 = 0

                # Abgrenzen der einzlnen Bereichsunterschiede
                x1 = np.linspace(interval_start_1, interval_end_1, N)
                x2 = np.linspace(interval_start_2, interval_end_2, N)
                x3 = np.linspace(interval_start_3, interval_end_3, N)

                # Definition der Spannungsfunktionen
                y1 = []
                y2 = []
                y3 = []
                # Grenzdehnung von 2 wird nicht erreicht
                if 0 <= epsc < 2.0:
                    y1_list_with_all_sigma_c_values_promille = []
                    for i in range(N):
                        sigma_c = fcd * (1 - (1 - ((x1[i] * epsc) / (x * 2))) ** 2)

                        y1_list_with_all_sigma_c_values_promille.append(sigma_c)

                    y1_values = y1_list_with_all_sigma_c_values_promille
                    y2_values = fcd * np.zeros(N)
                    y3_values = fcd * np.zeros(N)

                    y1.append(y1_values)
                    y2.append(y2_values)
                    y3.append(y3_values)
                # Grenzdehnung von 2 wird erreicht
                elif epsc >= 2.0:
                    y1_list_with_all_sigma_c_values_upto_2_promille = []
                    for i in range(N):
                        sigma_c = fcd * (
                            1 - (1 - ((x1[i] * epsc) / (x * 2))) ** 2
                        )  # Sigma c-Werte bis zum Erreichen der Druckfestigkeit -> Betondehnung = 2

                        y1_list_with_all_sigma_c_values_upto_2_promille.append(sigma_c)

                    y1_values = y1_list_with_all_sigma_c_values_upto_2_promille  # sigma_concrete_at_given_intervals(x1)
                    y2_values = fcd * np.ones(N)
                    y3_values = fcd * np.zeros(N)

                    y1.append(y1_values)
                    y2.append(y2_values)
                    y3.append(y3_values)

                # Integration der einzelnen Spannungsfunktionen
                if 0 <= epsc < 2.0:
                    M_1 = simps(y1 * x1, x1)
                    A_1 = simps(y1, x1)
                    S_1 = M_1 / A_1

                elif epsc >= 2.0:
                    M_1 = simps(y1 * x1, x1)  # moment 1
                    A_1 = simps(y1, x1)  # pressure force 1
                    S_1 = M_1 / A_1  # centroid of area 1

                    M_2 = simps(y2 * x2, x2)
                    A_2 = simps(y2, x2)
                    S_2 = M_2 / A_2

                # concrete pressure
                Fc_1 = A_1 * concrete_width_1
                Fc_2 = A_2 * concrete_width_2
                Fc_3 = 0

                # centroid of compressive stress in concrete
                S_ges = (S_1 * Fc_1 + S_2 * Fc_2 + S_3 * Fc_3) / (
                    Fc_1 + Fc_2 + Fc_3
                )  # Bei der Berechnung der Gesamtschwerpunkte der Druckkraefte muss die Querschnittsbreite der einzelnen Querschnitte einfliessen
                z = S_ges + d - x  # Innerer Hebelarm

                Meds = abs(M - N_f * (d - zs1))
                Mrds = (Fc_1 + Fc_2 + Fc_3) * z

                nu_eds = Meds / (b_w * (d**2) * fcd)
                if nu_eds_grenz <= nu_eds:
                    Meds_lim = nu_eds_grenz * b_w * (d**2) * fcd

                    zeta_lim = 0
                    concrete_strain_2 = 3.5
                    if nu_eds_grenz == 0.296:
                        zeta_lim = 0.813
                        steal_strain_1 = 4.28
                    elif nu_eds_grenz == 0.371:
                        zeta_lim = 0.743
                        steal_strain_1 = 2.174

                    # epss2 = ((x - d2) / x) * abs(-3.5)
                    epss2 = ((steal_strain_1 + concrete_strain_2) / d) * (
                        d - d2
                    ) - steal_strain_1

                    sigma_s_2 = 0
                    if epssyd <= epss2:
                        sigma_s_2 = fyd
                    elif epss2 < epssyd:
                        sigma_s_2 = fyd * (epss / epssyd)

                    z_nu_eds_grenz = zeta_lim * d

                    delta_Med = abs(M) - Meds_lim
                    As1erf_nu_lim = (
                        (1 / fyd) * (Meds_lim / z_nu_eds_grenz + N_f) * 100**2
                    )
                    As2erf = (1 / sigma_s_2) * (delta_Med / (d - d2)) * 100**2
                    As1erf = As1erf_nu_lim + As2erf
                else:
                    As1erf = (1 / sigma_s) * (Fc_1 + Fc_2 + Fc_3 + N_f) * 100**2

                if 0 <= epsc < 3.5:
                    epsc += 0.001

                else:
                    epss -= 0.001

                num_iter += 1
            # Stützmoment
            elif 0 > M:

                # Grenzdehnung von 2 wird nicht erreicht
                if 0 <= epsc < 2.0:
                    # Querschnittslängen Vergabe
                    interval_start_1 = 0
                    interval_end_1 = x
                    interval_start_2 = 0
                    interval_end_2 = 0
                    interval_start_3 = 0
                    interval_end_3 = 0
                    # Querschnittsbreiten Vergabe
                    concrete_width_1 = b_w
                    concrete_width_2 = 0
                    concrete_width_3 = 0
                # Grenzdehnung von 2 wird erreicht
                elif epsc >= 2.0:
                    # Querschnittslängen Vergabe
                    interval_start_1 = 0
                    interval_end_1 = (
                        2 / epsc * x
                    )  # 2 = Dehnung beim Erreichen der Druckfestigkeit
                    interval_start_2 = 2 / epsc * x
                    interval_end_2 = x
                    interval_start_3 = 0
                    interval_end_3 = 0
                    # Querschnittsbreiten Vergabe
                    concrete_width_1 = b_w
                    concrete_width_2 = b_w
                    concrete_width_3 = 0

                # Abgrenzen der einzlnen Bereichsunterschiede
                x1 = np.linspace(interval_start_1, interval_end_1, N)
                x2 = np.linspace(interval_start_2, interval_end_2, N)
                x3 = np.linspace(interval_start_3, interval_end_3, N)

                # Definition der Spannungsfunktionen
                y1 = []
                y2 = []
                y3 = []
                # Grenzdehnung von 2 wird nicht erreicht
                if 0 <= epsc < 2.0:
                    y1_list_with_all_sigma_c_values_promille = []
                    for i in range(N):
                        sigma_c = fcd * (1 - (1 - ((x1[i] * epsc) / (x * 2))) ** 2)

                        y1_list_with_all_sigma_c_values_promille.append(sigma_c)

                    y1_values = y1_list_with_all_sigma_c_values_promille
                    y2_values = fcd * np.zeros(N)
                    y3_values = fcd * np.zeros(N)

                    y1.append(y1_values)
                    y2.append(y2_values)
                    y3.append(y3_values)
                # Grenzdehnung von 2 wird erreicht
                elif epsc >= 2.0:
                    y1_list_with_all_sigma_c_values_upto_2_promille = []
                    for i in range(N):
                        sigma_c = fcd * (
                            1 - (1 - ((x1[i] * epsc) / (x * 2))) ** 2
                        )  # Sigma c-Werte bis zum Erreichen der Druckfestigkeit -> Betondehnung = 2

                        y1_list_with_all_sigma_c_values_upto_2_promille.append(sigma_c)

                    y1_values = y1_list_with_all_sigma_c_values_upto_2_promille  # sigma_concrete_at_given_intervals(x1)
                    y2_values = fcd * np.ones(N)
                    y3_values = fcd * np.zeros(N)

                    y1.append(y1_values)
                    y2.append(y2_values)
                    y3.append(y3_values)

                # Integration der einzelnen Spannungsfunktionen
                if 0 <= epsc < 2.0:
                    M_1 = simps(y1 * x1, x1)
                    A_1 = simps(y1, x1)
                    S_1 = M_1 / A_1

                elif epsc >= 2.0:
                    M_1 = simps(y1 * x1, x1)  # moment 1
                    A_1 = simps(y1, x1)  # pressure force 1
                    S_1 = M_1 / A_1  # centroid of area 1

                    M_2 = simps(y2 * x2, x2)
                    A_2 = simps(y2, x2)
                    S_2 = M_2 / A_2

                # concrete pressure
                Fc_1 = A_1 * concrete_width_1
                Fc_2 = A_2 * concrete_width_2
                Fc_3 = 0

                # centroid of compressive stress in concrete
                S_ges = (S_1 * Fc_1 + S_2 * Fc_2 + S_3 * Fc_3) / (
                    Fc_1 + Fc_2 + Fc_3
                )  # Bei der Berechnung der Gesamtschwerpunkte der Druckkraefte muss die Querschnittsbreite der einzelnen Querschnitte einfliessen
                z = S_ges + d - x  # Innerer Hebelarm

                Meds = abs(M - N_f * (zs1 - (h - d)))
                Mrds = (Fc_1 + Fc_2 + Fc_3) * z

                nu_eds = Meds / (b_w * (d**2) * fcd)
                if nu_eds_grenz <= nu_eds:
                    Meds_lim = nu_eds_grenz * b_w * (d**2) * fcd

                    zeta_lim = 0
                    concrete_strain_2 = 3.5
                    if nu_eds_grenz == 0.296:
                        zeta_lim = 0.813
                        steal_strain_1 = 4.28
                    elif nu_eds_grenz == 0.371:
                        zeta_lim = 0.743
                        steal_strain_1 = 2.174

                    # epss2 = ((x - d2) / x) * abs(-3.5)
                    epss2 = ((steal_strain_1 + concrete_strain_2) / d) * (
                        d - d2
                    ) - steal_strain_1

                    sigma_s_2 = 0
                    if epssyd <= epss2:
                        sigma_s_2 = fyd
                    elif epss2 < epssyd:
                        sigma_s_2 = fyd * (epss / epssyd)

                    z_nu_eds_grenz = zeta_lim * d

                    delta_Med = abs(M) - Meds_lim
                    As1erf_nu_lim = (
                        (1 / fyd) * (Meds_lim / z_nu_eds_grenz + N_f) * 100**2
                    )
                    As2erf = (1 / sigma_s_2) * (delta_Med / (d - d2)) * 100**2
                    As1erf = As1erf_nu_lim + As2erf
                else:
                    As1erf = (1 / sigma_s) * (Fc_1 + Fc_2 + Fc_3 + N_f) * 100**2

                if 0 <= epsc < 3.5:
                    epsc += 0.001
                else:
                    epss -= 0.001

                num_iter += 1

            else:
                print("Limit reached!")
                break

        # DIN 1045-1:2008-08
        amax = 0
        a1 = self.center_of_gravity
        a2 = h - self.center_of_gravity
        if a1 > a2:
            amax = a1 * 100
        else:
            amax = a2 * 100
        Wc = (self.cross_section_inertia * 100 * 100 * 100 * 100) / amax
        Asmin = ((fctm * 10**2) * Wc) / (fyk * 10**2 * (d * 0.9) * 100)

        needed_reinforcement = []
        if Asmin < As1erf:
            print("Iteration steps: ", num_iter)
            print("Med: ", np.around(M, 3), "MN")
            print("Ned: ", np.around(N_f, 3), "MN")
            print("Affecting moment: ", np.around(Meds, 3), "MN")
            print("Marginal moment: ", np.around(Meds_lim, 3), "MN")
            print("Modulus of section : ", np.around(Mrds, 3), "MN")
            print("Required bending reinforcement: ", np.around(As1erf, 3), "cm²")
            if As2erf != 0:
                print(
                    "Required compressive reinforcement : ", np.around(As2erf, 3), "cm²"
                )
            print("Pressure zone height x =  ", np.around(x, 3), "m")
            print("Retrieved pressure zone ξ = ", np.around(xi, 3))
            print("Edge concrete compression εc = ", np.around(epsc, 3), "‰")
            if epss >= 2.17:
                print(
                    "Steel strain of bending reinforcement εs: Yield strength is reached."
                )
                if As2erf != 0:
                    print("εs = ", np.around(steal_strain_1, 3), "‰")
                else:
                    print("εs = ", np.around(epss, 3), "‰")
            else:
                if As2erf != 0:
                    print(
                        "Steel strain of bending reinforcement εs: Yield strength is reached."
                    )
                    print("εs = ", np.around(steal_strain_1, 3), "‰")
                else:
                    print(
                        "Steel strain of bending reinforcement εs: Yield strength is not reached."
                    )
                    print("εs = ", np.around(epss, 3), "‰")

            if As2erf != 0:
                if As2erf != 0:
                    print(
                        "Steel strain of compressive reinforcement εs2: Yield strength is reached."
                    )
                    print("εs2 = ", np.around(epss2, 3), "‰")
                else:
                    print(
                        "Steel strain of compressive reinforcement εs2: Yield strength is not reached."
                    )
                    print("εs2 = ", np.around(epss2, 3), "‰")

            print("Inner lever arm z = ", np.around(z, 3), "m")

            reinforcement = [float(As1erf), float(As2erf)]
            needed_reinforcement.append(reinforcement)
        else:
            print("Minimum reinforcement is dominant!")
            print("Required bending reinforcement: ", np.around(Asmin, 3), "cm²")
            reinforcement = [float(Asmin), float(0.0)]
            needed_reinforcement.append(reinforcement)

        return needed_reinforcement

    def calculate_beam_section_without_shearreinforcement(
        self, beam_sub_section, as1, strength_of_concrete, tensile_yield_strength
    ):
        """
        Function to calculate the resistance of a rectangular cross section without 
        a shear reinformcent (e.g. plates) \n 

        Args:
            beam_sub_section (_type_): _description_
            as1 (_type_): Biegezugbewehrung in [m**2]
            strength_of_concrete (_type_): _description_
            tensile_yield_strength (_type_): _description_
        """
        Ved = beam_sub_section.shear_force_ed / 1000000
        Vedr = abs(Ved)  # reduzierte Querkraft, da direkte Lagerung
        N_f = beam_sub_section.normal_force_ed / 1000000

        A = self.cross_section_area  # QS-Fläche

        h = self.height
        b = self.width  # kleinste QS-breite -> bw aus Biegebemessung

        d1 = beam_sub_section.effective_height_pressure  # cnom + dsl*0.5
        d = h - d1
        z = 0.9 * d

        fcd = strength_of_concrete  # N/mm²
        fck = (fcd * 1.5) / 0.85
        fctm = 0.3 * fck ** (2 / 3)
        yc = 1.5

        fyd = tensile_yield_strength  # MN/m²
        fyk = fyd * 1.15

        rho_l = as1 / (b * d)
        k = min(1 + np.sqrt(200 / (d * 1000)), 2)

        sigma_cd = min(N_f / A, 0.2 * fcd)  # Zugspannungen negativ

        self.v_rdcs = (
            (0.15 / 1.5 * k * (100 * rho_l * fck) ** (1 / 3) + 0.12 * sigma_cd) * b * d
        )

        if d * 1000 < 600:
            self.nu_rd_c_min = 0.0525 / 1.5 * k**1.5 * fck**0.5
        elif 600 > d * 1000 and d * 1000 <= 800:
            nu = 0.0525 - (0.0525 - 0.0375) / 200 * (d * 1000 - 600)
            self.nu_rd_c_min = nu / 1.5 * k**1.5 * fck**0.5
        elif d * 1000 > 800:
            self.nu_rd_c_min = 0.0375 / 1.5 * k**1.5 * fck**0.5

        self.v_rdcmin = (self.nu_rd_c_min + 0.12 * sigma_cd) * b * d

    # Shear-Force Calculation after DIN EN 1992-1-1
    def calculate_beam_section_stirrup(
        self,
        beam_sub_section,
        strength_of_concrete,
        tensile_yield_strength,
        strut_angle_choice,
    ):
        Ved = beam_sub_section.shear_force_ed / 1000000
        Vedr = abs(Ved)  # reduzierte Querkraft, da direkte Lagerung
        N_f = beam_sub_section.normal_force_ed / 1000000

        A = self.cross_section_area  # QS-Fläche

        h = self.height
        b = self.width  # kleinste QS-breite -> bw aus Biegebemessung

        d1 = beam_sub_section.effective_height_pressure  # cnom + dsl*0.5
        d = h - d1
        z = 0.9 * d

        fcd = strength_of_concrete  # N/mm²
        fck = (fcd * 1.5) / 0.85
        fctm = 0.3 * fck ** (2 / 3)
        yc = 1.5

        fyd = tensile_yield_strength  # MN/m²
        fyk = fyd * 1.15

        a_sw_correct = 0

        # Berechnung des Druckstrebenwinkels
        c = 0.5
        sigma_cd = N_f / A
        Vrd_cc = c * 0.48 * (fck ** (1 / 3)) * (1 - 1.2 * (sigma_cd / fcd)) * b * z

        cot_teta = abs((1.2 + 1.4 * (sigma_cd / fcd)) / (1 - (Vrd_cc / Vedr)))
        strut_angle = math.degrees(math.atan(1 / cot_teta))

        if strut_angle_choice == 0:
            if 1.0 <= cot_teta <= 3.0:
                cot_teta = cot_teta
            elif 3.0 < cot_teta:
                cot_teta = 3.0
                strut_angle = 18.43
            elif 0 <= cot_teta < 1.0:
                cot_teta = 1.0
                strut_angle = 45
        else:
            if N_f > 0:
                cot_teta = 1.0
                strut_angle = 45
            else:
                cot_teta = 1.2
                strut_angle = 40

        # Nachweis der Druckstrebentragfähigkeit
        alpha_cw = 1.0
        v2 = 1.1 - (fck / 500)
        v1 = 0.75 * v2
        tan_teta = math.tan(math.radians(strut_angle))
        Vrd_max = (alpha_cw * b * z * v1 * fcd) / ((tan_teta) + (cot_teta))
        proof = abs(Vedr / Vrd_max)

        # Ermittlung der stat. erforderlichen Bewehrung, für vertikale Bügel
        a_sw = (Vedr / (z * fyd)) * tan_teta * 100**2

        # Konstruktive Mindestbewehrung
        a_sw_min = (
            0.16 * (fctm / fyk) * b * 1.0 * 100**2
        )  # Mindestquerkraftbewehrung, vertikale Bügel

        needed_reinforcement = []

        if a_sw <= a_sw_min:
            a_sw_correct = a_sw_min
        else:
            a_sw_correct = a_sw

        if proof <= 1.0:
            print(
                "Proof that the strut sturdiness is complied =  ",
                np.around(proof, 3),
                "%",
            )
        else:
            print(
                "Proof that the strut sturdiness is NOT complied =  ",
                np.around(proof, 3),
                "%",
            )

        print("Compression strut angle =  ", np.around(strut_angle, 2), "°")

        if a_sw <= a_sw_min:
            print(
                "Minimum reinforcement is required for the shear force =  ",
                np.around(a_sw_correct, 3),
                " cm²/m",
            )
        else:
            print(
                "Required shear force reinforcement =  ",
                np.around(a_sw_correct, 3),
                " cm²/m",
            )

        needed_reinforcement.append(float(np.around(a_sw_correct, 3)))

        return needed_reinforcement



BeamRectangular(1,1,0.20)




import numpy as np
import matplotlib.pyplot as plt

from pylatex import (
    Document,
    Section,
    Subsection,
    Command,
    Tabular,
    Math,
    TikZ,
    Axis,
    Plot,
    Figure,
    Matrix,
    Alignat,
    MultiRow,
    MultiColumn, 
    Tabularx,
    Tabu,
    ColumnType,
    LongTable,
    Table, 
    Center

)

from pylatex.utils import escape_latex, fix_filename, dumps_list, bold, \
    italic, verbatim, NoEscape



class Beulnachweise_Platten:
    """
    Das ist eine Klasse zur Berechnung einfacher Beulfelder.

    self.sigma_1_x =    Größte Druckspannung an einer Seite des Beulfeldes.
                        Hier wird üblicherweise ein positives Vorzeichen        
                        für Druckspannungen gewählt.



    """
    def __init__(self):
        # Spannungen aus Sofistik
        self.sigma_1_x = 387.5       # Druckspannung (Positiv)
        self.sigma_2_x = -262.9      # Zugspannung negativ
        self.sigma_1_v = 387.5
        self.sigma_2_v = 262.9
        self.tau_1 = 0
        self.tau_2 = 0

        # Materialkenngrößen
        self.fyd = 460
        self.E_s = 210e3 

        # Faktoren bis zum Erreichen der kritischen Beulvergleichsspannungen
        self.alpha_ult_k_1_x = self.fyd/self.sigma_1_x
        # Geometrie des Beulfeldes
        self.b = 5.00
        self.a = 1.90
        self.t = 0.02
        self.alpha = self.a/self.b

        self.chi_w  = 0
        self.rho    = 0

    def Nachweisfuehrung(self):
        # Ermittlung linear elastischer Beulspannungen

        self.sigma_e = 1.9*(100*self.t/self.b)**2*10
        self.psi = self.sigma_2_x/self.sigma_1_x       # Randspannungsverhältnis bezogen auf die größe Druckspannung (Druck positiv)

        self.k_sigma = (self.alpha + 1/self.alpha)**2 * (2.1)/(self.psi+1.1)

        self.sigma_cr = self.k_sigma*self.sigma_e
        self.alpha_cr = self.sigma_cr/self.sigma_1_x

        self.alpha_ult_k = self.fyd/self.sigma_1_v
    
        #____Abminderungsfaktoren für das Plattenbeulen für ein beidseitig gestützes Feld___#
        self.lambda_p = np.sqrt(self.alpha_ult_k/self.alpha_cr)

        # Abminderungsfaktor chi_w

        self.eta = 1.20
        if (self.fyd > 460):
            self.eta = 1.00
        if (self.lambda_p < 0.83/self.eta):
            self.chi_w = self.eta
        elif(self.lambda_p >= 0.83 / self.eta):
            self.chi_w = 0.83 / self.lambda_p
        
        print(self.chi_w)

        # Abminderungskenngröße rho_x
        # Für plattenartiges Versagen

        if (self.lambda_p <= 0.5 + np.sqrt(0.085 - 0.055 * self.psi)):
            self.rho = 1.0
        elif (self.lambda_p > 0.5 + np.sqrt(0.085 - 0.055 * self.psi)):
            self.rho = min((self.lambda_p - 0.055 * (3+self.psi))/(self.lambda_p**2),1.00)

        # Für Knickstabähnliches Verhalten

        self.alpha_e    = 0.21 # Für nicht ausgesteifte Blechfelder: alpha_e = 0.21

        self.phi        = 0.5*(1+self.alpha_e*(self.lambda_p -0.2) + self.lambda_p**2)

        self.chi_c      = 1/(self.phi + np.sqrt(self.phi**2 - self.lambda_p**2))

        # Knickspannung

        self.sigma_cr_r = (np.pi**2 * self.E_s * self.t**2)/(12 * (1-0.3)*self.a**2)
        self.sigma_cr_p_x = self.sigma_1_x

        # Interpolation zwischen plattenartigen / knickstabähnlichem Verhalten

        self.xi = self.sigma_cr_p_x/self.sigma_cr_r-1

        print(self.xi)

        self.xi = max(min(self.xi,1),0)

        print(self.xi)

        self.rho_x = (self.rho - self.chi_c)*self.xi*(2-self.xi) + self.chi_c

        print(self.rho_x)


        # Nachweisführung

        self.eta = (self.sigma_1_x/(self.fyd * self.rho_x / 1.10))

    def Ausgabe_Pdf_Latex(self,width ,*args, **kwargs):
        fname = "Beulnachweise"
        geometry_options = {"right": "2cm", "left": "2cm"}
        self.doc = Document(fname, geometry_options=geometry_options)

        with self.doc.create(Section("Berechnung der Beulnachweise nach DIN EN 1993-1-5")):
            self.doc.append("Take a look at this beautiful plot:")

            # with doc.create(Figure(position="htbp")) as plot:
            #     plot.add_plot(width=NoEscape(width), *args, **kwargs)
            #     plot.add_caption("I am a caption.")

            self.doc.create(Subsection("Materialkenngrößen"))
            self.doc.append("")
            #self.doc.append(Math(data=["2*3", "=", 3 * 2]))

        self.doc.create(Subsection("Geometrische und materielle Kenngrößen"))

        def Material_Properties():
            # Tabular

            self.doc.append(NoEscape(r"\centering"))
            
            table = Table(position="htb")

            self.doc.append(Command('centering'))
            table.add_caption("Material properties")


            t = Tabular(table_spec='|c|c|c|', data=None, pos=1, width=3)

            t.add_hline(start=None, end=None)

            t.add_row(("Kenngröße", "Wert", "Einheit"), escape=False, strict=True, mapper=[bold])
            t.add_hline(start=None, end=None)

            t.add_row(["Streckgrenze", self.fyd, "MN/m²"])
            t.add_row([NoEscape(r"Streckgrenze - $$\gamma_{\text{M1}}$$") , f"{self.fyd/1.10:.2f}", "MN/m²"])
            t.add_row(["E-Modul", self.E_s, "MN/m²"])
            t.add_hline(start=None, end=None)
            # t.add_row(1, 2, escape=False, strict=True, mapper=[bold])

            # # MultiColumn/MultiRow.
            # t.add_row((MultiColumn(size=2, align='|c|', data='MultiColumn'),),
            #         strict=True)

            # # One multiRow-cell in that table would not be proper LaTeX,
            # # so strict is set to False

            # t.add_row((MultiRow(size=2, width='*', data='MultiRow'),), strict=False)

            # append tabular to table
            table.append(t)

            table = self.doc.append(table)



        Material_Properties()

        def Section_Entry_Parameters():
            
            table = Table(position="htb")

            
            table.add_caption("Eingangsgrößen")

            t = Tabular(table_spec='|c|c|c|', data=None, pos=1, width=3)
            self.doc.append(Command('centering'))

            t.add_hline(start=None, end=None)

            t.add_row(("Kenngröße", "Wert", "Einheit"), escape=False, strict=True, mapper=[bold])
            t.add_hline(start=None, end=None)

            t.add_row(["Länge des Beulfeldes a", self.a, "m"])
            t.add_row(["Breite des Beulfeldes a", self.b, "m"])
            t.add_row(["Blechdicke t ", self.t, "m"])

            t.add_row([NoEscape(r"Seitenverhältnis $$\alpha $$"), f"{self.a/self.b:.2f}", "-"])
            t.add_row([NoEscape(r"Spannungsverhältnis  $$\psi $$"), f"{self.psi:.2f}", "-"])

            t.add_hline(start=None, end=None)
            # t.add_row(1, 2, escape=False, strict=True, mapper=[bold])

            # # MultiColumn/MultiRow.
            # t.add_row((MultiColumn(size=2, align='|c|', data='MultiColumn'),),
            #         strict=True)

            # # One multiRow-cell in that table would not be proper LaTeX,
            # # so strict is set to False

            # t.add_row((MultiRow(size=2, width='*', data='MultiRow'),), strict=False)

            # append tabular to table
            table.append(t)

            table = self.doc.append(table)

        Section_Entry_Parameters()

        self.doc.create(Subsection("Abminderungsfaktoren für die Streckgrenze"))

        def Abschnitt_Beulwerte():
            self.doc.append(NoEscape(f"Die Beulwerte werden zunächst in Abhängigkeit des Seitenverhältnisses {{\alpha}} bestimmt. "))
            self.doc.append(NoEscape(f"Für Seitenverhältnisse <= 1  sind die Beulwerte (rechnerisch) unabhängig vom Seitenverhältnis."))

                                
            if (self.alpha < 1.0):
                self.doc.append(Math(data=[NoEscape(f"k_{{\sigma}} = ({self.alpha:.2f} + 1/{self.alpha:.2f} ){{^2}} {{\cdot}} 2.1/({self.psi:.2f} +1.1) = {self.k_sigma:.2f}")])) 
            elif (self.alpha > 1.0):
                if (self.psi == -1):
                    self.doc.append(Math(data=[23.9]))
                elif(self.psi <0 and self.psi > -1):
                    self.res_biegung_normal = 7.81 - 6.29 * self.psi + 9.78 * self.psi**2
                    self.doc.append(Math(data=[NoEscape(f"k_{{\sigma}} = 7.81 - 6.29 {{\cdot \psi}} + 9.78 {{\cdot \psi^2 }} = {self.res_biegung_normal}")]))

        Abschnitt_Beulwerte()







        self.doc.generate_pdf(clean_tex=False)




        

# a = Beulnachweise_Platten()
# a.Nachweisfuehrung()
# a.Ausgabe_Pdf_Latex(10)

        

    



from KIB_LAP.Betonbau import BeamRectangular
from KIB_LAP.Betonbau import beam_sub_section

import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
import pandas as pd
import time
import os


